@@ -59,6 +59,8 @@ if ($opt_V) {
 if ($opt_h) {print_help(); exit $ERRORS{'OK'};}
 
 my $smbclient = $utils::PATH_TO_SMBCLIENT;
+$smbclient    || usage("check requires smbclient, smbclient not set\n");
+-x $smbclient || usage("check requires smbclient, $smbclient: $!\n");
 
 # Options checking
 
@@ -176,7 +178,7 @@ $SIG{'ALRM'} = sub {
 };
 alarm($TIMEOUT);
 
-# Execute an "ls" on the share using smbclient program
+# Execute a "du" on the share using smbclient program
 # get the results into $res
 my @cmd = (
 	$smbclient,
@@ -185,7 +187,7 @@ my @cmd = (
 	defined($workgroup) ? ("-W", $workgroup) : (),
 	defined($address) ? ("-I", $address) : (),
 	defined($opt_P) ? ("-p", $opt_P) : (),
-	"-c", "ls"
+	"-c", "du"
 );
 
 print join(" ", @cmd) . "\n" if ($verbose);
@@ -198,7 +200,7 @@ alarm(0);
 @lines = split /\n/, $res;
 
 #Get the last line into $_
-$_ = $lines[$#lines];
+$_ = $lines[$#lines-1];
 #print "$_\n";
 
 #Process the last line to get free space.  
