@@ -1,5 +1,5 @@
 #!/usr/bin/perl -w
-
+#
 # (c)1999 Ian Cass, Knowledge Matters Ltd.
 # Read the GNU copyright stuff for all the legalese
 #
@@ -7,8 +7,7 @@
 # be installed on the system, however since it's part of the ntp suite, you 
 # should already have it installed.
 #
-# $Id: check_ntp.pl,v 1.20 2004/11/19 03:59:43 mattkent Exp $
-# 
+#
 # Nothing clever done in this program - its a very simple bare basics hack to
 # get the job done.
 #
@@ -61,8 +60,8 @@ require 5.004;
 use POSIX;
 use strict;
 use Getopt::Long;
-use vars qw($opt_V $opt_h $opt_H $opt_t $opt_w $opt_c $opt_j $opt_k $verbose $PROGNAME $def_jitter);
-use lib utils.pm; 
+use vars qw($opt_V $opt_h $opt_H $opt_t $opt_w $opt_c $opt_O $opt_j $opt_k $verbose $PROGNAME $def_jitter $ipv4 $ipv6);
+use lib utils.pm;
 use utils qw($TIMEOUT %ERRORS &print_revision &support);
 
 $PROGNAME="check_ntp";
@@ -85,16 +84,19 @@ Getopt::Long::Configure('bundling');
 GetOptions
 	("V"   => \$opt_V, "version"    => \$opt_V,
 	 "h"   => \$opt_h, "help"       => \$opt_h,
-	 "v" => \$verbose, "verbose"  	=> \$verbose,
+	 "v"   => \$verbose, "verbose" 	=> \$verbose,
+	 "4"   => \$ipv4, "use-ipv4"	=> \$ipv4,
+	 "6"   => \$ipv6, "use-ipv6"	=> \$ipv6,
 	 "w=f" => \$opt_w, "warning=f"  => \$opt_w,   # offset|adjust warning if above this number
 	 "c=f" => \$opt_c, "critical=f" => \$opt_c,   # offset|adjust critical if above this number
+	 "O"   => \$opt_O, "zero-offset" => \$opt_O,  # zero-offset  bad
 	 "j=s" => \$opt_j, "jwarn=i"    => \$opt_j,   # jitter warning if above this number
 	 "k=s" => \$opt_k, "jcrit=i"    => \$opt_k,   # jitter critical if above this number
 	 "t=s" => \$opt_t, "timeout=i"  => \$opt_t,
 	 "H=s" => \$opt_H, "hostname=s" => \$opt_H);
 
 if ($opt_V) {
-	print_revision($PROGNAME,'$Revision: 1.20 $ ');
+	print_revision($PROGNAME,'@NP_VERSION@');
 	exit $ERRORS{'OK'};
 }
 
@@ -177,6 +179,18 @@ $SIG{'ALRM'} = sub {
 };
 alarm($timeout);
 
+# Determine protocol to be used for ntpdate and ntpq
+my $ntpdate = $utils::PATH_TO_NTPDATE;
+my $ntpq    = $utils::PATH_TO_NTPQ;
+if ($ipv4) {
+        $ntpdate .= " -4";
+        $ntpq .= " -4";
+}
+elsif ($ipv6) {
+        $ntpdate .= " -6";
+        $ntpq .= " -6";
+}
+# else don't use any flags
 
 ###
 ###
@@ -184,14 +198,16 @@ alarm($timeout);
 ###
 ###
 
-if (!open (NTPDATE, "$utils::PATH_TO_NTPDATE -q $host 2>&1 |")) {
-	print "Could not open ntpdate\n";
+if (!open (NTPDATE, $ntpdate . " -q $host 2>&1 |")) {
+	print "Could not open $ntpdate: $!\n";
 	exit $ERRORS{"UNKNOWN"};
 }
 
+my $out;
 while (<NTPDATE>) {
-	print if ($verbose);
+	#print if ($verbose);  # noop
 	$msg = $_ unless ($msg);
+	$out .= "$_ ";
 	
 	if (/stratum\s(\d+)/) {
 		$stratum = $1;
@@ -202,7 +218,11 @@ while (<NTPDATE>) {
 
 		# An offset of 0.000000 with an error is probably bogus. Actually,
 		# it's probably always bogus, but let's be paranoid here.
-		if ($offset == 0) { undef $offset;}
+		# Has been reported that 0.0000 happens in a production environment
+		# on Solaris 8 so this check should be taken out - SF tracker 1150777
+		if (defined $opt_O ) {
+			if ($offset == 0) { undef $offset;}
+		}
 
 		$ntpdate_error = defined ($offset) ? $ERRORS{"OK"} : $ERRORS{"CRITICAL"};
 		print "ntperr = $ntpdate_error \n" if $verbose;
@@ -222,8 +242,11 @@ while (<NTPDATE>) {
 	}
 
 }
+$out =~ s/\n//g;
+close (NTPDATE) || 
+    die $! ? "$out - Error closing $ntpdate pipe: $!"
+           : "$out - Exit status: $? from $ntpdate\n";
 
-close (NTPDATE); 
 # declare an error if we also get a non-zero return code from ntpdate
 # unless already set to critical
 if ( $? && !$ignoreret ) {
@@ -259,7 +282,7 @@ if ( $? && !$ignoreret ) {
 
 if ($have_ntpq) {
 
-	if ( open(NTPQ,"$utils::PATH_TO_NTPQ -np $host 2>&1 |") ) {
+	if ( open(NTPQ, $ntpq . " -np $host 2>&1 |") ) {
 		while (<NTPQ>) {
 			print $_ if ($verbose);
 			if ( /timed out/ ){
@@ -270,11 +293,11 @@ if ($have_ntpq) {
 			if (/^(\*|\+|\#|o])/) {
 				++$candidate;
 				push (@candidates, $_);
-				print "Candiate count= $candidate\n" if ($verbose);
+				print "Candidate count= $candidate\n" if ($verbose);
 			}
 			
 			# match sys.peer or pps.peer
-			if (/^(\*|o)([-0-9.\s]+)\s+([-0-9A-Za-z.]+)\s+([-0-9.]+)\s+([lumb-]+)\s+([-0-9m.]+)\s+([-0-9.]+)\s+([-0-9.]+)\s+([-0-9.]+)\s+([-0-9.]+)\s+([-0-9.]+)/) {
+			if (/^(\*|o)(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)/) {
 				$syspeer = $2;
 				$stratum = $4;
 				$jitter = $11;
@@ -283,23 +306,28 @@ if ($have_ntpq) {
 					print "Jitter_crit = $11 :$jcrit\n" if ($verbose);
 					$jitter_error = $ERRORS{'CRITICAL'};
 				} elsif ($jitter > $jwarn ) {
-					print "Jitter_warn = $11 :$jwarn \n" if ($verbose);
+					print "Jitter_warn = $11 :$jwarn\n" if ($verbose);
 					$jitter_error = $ERRORS{'WARNING'};
 				} else {
 					$jitter_error = $ERRORS{'OK'};
 				}
+			} else {
+				print "No match!\n" if $verbose;
+				$jitter = '(not parsed)';
 			}
 			
 		}
-		close NTPQ;
+		close NTPQ ||
+            die $! ? "Error closing $ntpq pipe: $!"
+                   : "Exit status: $? from $ntpq\n";
 
 		# if we did not match sys.peer or pps.peer but matched # candidates only
 		# generate a warning 
 		# based on bug id 773588
 		unless (defined $syspeer) {
-			if ($#candidates >0) {
+			if ($#candidates >=0) {
 				foreach my $c (@candidates) {
-					$c =~ /^(#)([-0-9.\s]+)\s+([-0-9A-Za-z.]+)\s+([-0-9.]+)\s+([lumb-]+)\s+([-0-9m.]+)\s+([-0-9.]+)\s+([-0-9.]+)\s+([-0-9.]+)\s+([-0-9.]+)\s+([-0-9.]+)/;
+					$c =~ /^(#)([-0-9.\s]+)\s+([-0-9A-Za-z_().]+)\s+([-0-9.]+)\s+([lumb-]+)\s+([-0-9m.]+)\s+([-0-9.]+)\s+([-0-9.]+)\s+([-0-9.]+)\s+([-0-9.]+)\s+([-0-9.]+)/;
 					$syspeer = $2;
 					$stratum = $4;
 					$jitter = $11;
@@ -324,53 +352,53 @@ if ($have_ntpq) {
 if ($ntpdate_error != $ERRORS{'OK'}) {
 	$state = $ntpdate_error;
 	if ($ntpdate_error == $ERRORS{'WARNING'} ) {
-		$answer = $msg . "\n";
+		$answer = $msg;
 	}
 	else {
-		$answer = $msg . "Server for ntp probably down\n";
+		$answer = $msg . "Server for ntp probably down";
 	}
 
 	if (defined($offset) && abs($offset) > $ocrit) {
 		$state = $ERRORS{'CRITICAL'};
-		$answer = "Server Error and offset $offset sec > +/- $ocrit sec\n";
+		$answer = "Server Error and offset $offset sec > +/- $ocrit sec";
 	} elsif (defined($offset) && abs($offset) > $owarn) {
-		$answer = "Server error and offset $offset sec > +/- $owarn sec\n";
+		$answer = "Server error and offset $offset sec > +/- $owarn sec";
 	} elsif (defined($jitter) && abs($jitter) > $jcrit) {
-		$answer = "Server error and jitter $jitter msec > +/- $jcrit msec\n";
+		$answer = "Server error and jitter $jitter msec > +/- $jcrit msec";
 	} elsif (defined($jitter) && abs($jitter) > $jwarn) {
-		$answer = "Server error and jitter $jitter msec > +/- $jwarn msec\n";
+		$answer = "Server error and jitter $jitter msec > +/- $jwarn msec";
 	}
 
 } elsif ($have_ntpq && $jitter_error != $ERRORS{'OK'}) {
 	$state = $jitter_error;
-	$answer = "Jitter $jitter too high\n";
+	$answer = "Jitter $jitter too high";
 	if (defined($offset) && abs($offset) > $ocrit) {
 		$state = $ERRORS{'CRITICAL'};
-		$answer = "Jitter error and offset $offset sec > +/- $ocrit sec\n";
+		$answer = "Jitter error and offset $offset sec > +/- $ocrit sec";
 	} elsif (defined($offset) && abs($offset) > $owarn) {
-		$answer = "Jitter error and offset $offset sec > +/- $owarn sec\n";
+		$answer = "Jitter error and offset $offset sec > +/- $owarn sec";
 	} elsif (defined($jitter) && abs($jitter) > $jcrit) {
-		$answer = "Jitter error and jitter $jitter msec > +/- $jcrit msec\n";
+		$answer = "Jitter error and jitter $jitter msec > +/- $jcrit msec";
 	} elsif (defined($jitter) && abs($jitter) > $jwarn) {
-		$answer = "Jitter error and jitter $jitter msec > +/- $jwarn msec\n";
+		$answer = "Jitter error and jitter $jitter msec > +/- $jwarn msec";
 	}
 
 } elsif( !$have_ntpq ) { # no errors from ntpdate and no ntpq or ntpq timed out
 	if (abs($offset) > $ocrit) {
 		$state = $ERRORS{'CRITICAL'};
-		$answer = "Offset $offset sec > +/- $ocrit sec\n";
+		$answer = "Offset $offset sec > +/- $ocrit sec";
 	} elsif (abs($offset) > $owarn) {
 		$state = $ERRORS{'WARNING'};
-		$answer = "Offset $offset sec > +/- $owarn sec\n";
+		$answer = "Offset $offset sec > +/- $owarn sec";
 	} elsif (( abs($offset) > $owarn) && $def_jitter ) {
 		$state = $ERRORS{'WARNING'};
-		$answer = "Offset $offset sec > +/- $owarn sec, ntpq timed out\n";
+		$answer = "Offset $offset sec > +/- $owarn sec, ntpq timed out";
 	} elsif ( $def_jitter ) {
 		$state = $ERRORS{'WARNING'};
-		$answer = "Offset $offset secs, ntpq timed out\n";
+		$answer = "Offset $offset secs, ntpq timed out";
 	} else{
 		$state = $ERRORS{'OK'};
-		$answer = "Offset $offset secs \n";
+		$answer = "Offset $offset secs";
 	}
 
 
@@ -378,27 +406,28 @@ if ($ntpdate_error != $ERRORS{'OK'}) {
 } else { # no errors from ntpdate or ntpq
 	if (abs($offset) > $ocrit) {
 		$state = $ERRORS{'CRITICAL'};
-		$answer = "Offset $offset sec > +/- $ocrit sec, jitter $jitter msec\n";
+		$answer = "Offset $offset sec > +/- $ocrit sec, jitter $jitter msec";
 	} elsif (abs($jitter) > $jcrit ) {
 		$state = $ERRORS{'CRITICAL'};
-		$answer = "Jitter $jitter msec> +/- $jcrit msec, offset $offset sec \n";
+		$answer = "Jitter $jitter msec> +/- $jcrit msec, offset $offset sec";
 	} elsif (abs($offset) > $owarn) {
 		$state = $ERRORS{'WARNING'};
-		$answer = "Offset $offset sec > +/- $owarn sec, jitter $jitter msec\n";
+		$answer = "Offset $offset sec > +/- $owarn sec, jitter $jitter msec";
 	} elsif (abs($jitter) > $jwarn ) {
 		$state = $ERRORS{'WARNING'};
-		$answer = "Jitter $jitter msec> +/- $jwarn msec, offset $offset sec \n";
+		$answer = "Jitter $jitter msec> +/- $jwarn msec, offset $offset sec";
 
 	} else {
 		$state = $ERRORS{'OK'};
-		$answer = "Offset $offset secs, jitter $jitter msec, peer is stratum $stratum\n";
+		$answer = "Offset $offset secs, jitter $jitter msec, peer is stratum $stratum";
 	}
 	
 }
 
 foreach my $key (keys %ERRORS) {
 	if ($state==$ERRORS{$key}) {
-		print ("NTP $key: $answer");
+#		print ("NTP $key: $answer");
+		print ("NTP $key: $answer|offset=$offset, jitter=" . $jitter/1000 .	",peer_stratum=$stratum\n");
 		last;
 	}
 }
@@ -409,28 +438,34 @@ exit $state;
 #### subs
 
 sub print_usage () {
-	print "Usage: $PROGNAME -H <host> [-w <warn>] [-c <crit>] [-j <warn>] [-k <crit>] [-v verbose]\n";
+	print "Usage: $PROGNAME -H <host> [-46] [-O] [-w <warn>] [-c <crit>] [-j <warn>] [-k <crit>] [-v verbose]\n";
 }
 
 sub print_help () {
-	print_revision($PROGNAME,'$Revision: 1.20 $');
+	print_revision($PROGNAME,'@NP_VERSION@');
 	print "Copyright (c) 2003 Bo Kersey/Karl DeBisschop\n";
 	print "\n";
 	print_usage();
 	print "
 Checks the local timestamp offset versus <host> with ntpdate
 Checks the jitter/dispersion of clock signal between <host> and its sys.peer with ntpq\n
--w ( --warning)
+-O (--zero-offset)
+     A zero offset on \"ntpdate\" will generate a CRITICAL.\n
+-w (--warning)
      Clock offset in seconds at which a warning message will be generated.\n	Defaults to $DEFAULT_OFFSET_WARN.
 -c (--critical) 
      Clock offset in seconds at which a critical message will be generated.\n	Defaults to $DEFAULT_OFFSET_CRIT.
 -j (--jwarn)
      Clock jitter in milliseconds at which a warning message will be generated.\n	Defaults to $DEFAULT_JITTER_WARN.
 -k (--jcrit)
-    Clock jitter in milliseconds at which a warning message will be generated.\n	Defaults to $DEFAULT_JITTER_CRIT.\n
+    Clock jitter in milliseconds at which a critical message will be generated.\n	Defaults to $DEFAULT_JITTER_CRIT.
     
     If jitter/dispersion is specified with -j or -k and ntpq times out, then a
-    warning is returned.
-";	
+    warning is returned.\n
+-4 (--use-ipv4)
+    Use IPv4 connection
+-6 (--use-ipv6)
+    Use IPv6 connection
+\n";	
 support();
 }
