@@ -35,6 +35,8 @@ const char *email = "nagiosplug-devel@lists.sourceforge.net";
 
 #define DEFAULT_TIMEOUT 11
 
+#include <ctype.h>
+
 #include "common.h"
 #include "utils.h"
 #include "utils_cmd.h"
@@ -43,6 +45,7 @@ const char *email = "nagiosplug-devel@lists.sourceforge.net";
 
 static const char **process_arguments (int, char **);
 int validate_arguments (char **);
+int translate_state (char *);
 void print_help (void);
 void print_usage (void);
 int subst_text = FALSE;
@@ -104,7 +107,7 @@ main (int argc, char **argv)
 				*sub = '\0';
 				sub += strlen (state_text (result));
 				/* then put everything back together */
-				asprintf (&chld_out.line[i], "%s%s%s", chld_out.line[i], state_text (state[result]), sub);
+				xasprintf (&chld_out.line[i], "%s%s%s", chld_out.line[i], state_text (state[result]), sub);
 			}
 		}
 		printf ("%s\n", chld_out.line[i]);
